
/* Network traffic analyzer
 * (c) 2001-2002, Emil Mikulic.
 */

/* big numbers - "arbitrary precision" arithmetic using strings */
#include "bignum.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#ifndef HAVE_QUAD

inline dword num_from_char(const byte c)
{
	return (dword)(c - '0');
}



/* note: output needs to be deallocated */
inline char *bignum_from_int(const dword n)
{
	char *s;
       	int len = 1;

	/*    4294967296 */
	if (n>999999999) len = 10; else
	if (n>99999999) len = 9; else
	if (n>9999999) len = 8; else
	if (n>999999) len = 7; else
	if (n>99999) len = 6; else
	if (n>9999) len = 5; else
	if (n>999) len = 4; else
	if (n>99) len = 3; else
	if (n>9) len = 2;

	s = (char*)malloc(len+1);
	sprintf(s, "%u", n);

	return s;
}



/* note: output needs to be deallocated */
inline char *add_bignum(char *a, char *b)
{
	char *s_s, *s_l, buf[1024], *ret;
	int i, carry;
	
	if (strlen(a)>strlen(b)) 
	{
		s_l = a;
		s_s = b;
	} else {
		s_s = a;
		s_l = b;
	}
		
	carry = 0;
	for (i=1; i<=strlen(s_l); i++)
	{
		char x,y;
		int ix, iy, iz;

		x = s_l[strlen(s_l)-i];
		if ((signed int)(strlen(s_s))-i >= 0) 
			y = s_s[strlen(s_s)-i];
		else
			y = '0';

		ix = num_from_char(x);
		iy = num_from_char(y);
		iz = ix + iy + carry;
		if (iz > 9)
		{
			int tmp = iz % 10;
			carry = (iz - tmp) / 10;
			iz = tmp;
		} else carry = 0;
	
		buf[i-1] = '0' + iz;
	}

	i = strlen(s_l);
	if (carry > 0) {
		buf[i] = '0' + carry;
		buf[i+1] = 0;
	} else {
		buf[i] = 0;
	}

	ret = (char*)malloc(strlen(buf)+1);
	for (i=0; i<strlen(buf); i++)
	{
		ret[strlen(buf)-i-1] = buf[i];
	}
	ret[strlen(buf)] = 0;

	return ret;
}



/* note: output needs to be deallocated */
inline char *mul_bignum(char *a, char *b)
{
	char *s_s, *s_l, buf[1024], buf2[1024], *ret, *ret2;
	int i,j, carry;

	if ((strcmp(a,"0") == 0) || (strcmp(b,"0") == 0))
		return strdup("0");

	if (strlen(a)>strlen(b)) 
	{
		s_l = a;
		s_s = b;
	} else {
		s_s = a;
		s_l = b;
	}

	ret = strdup("0");

	for (i=1; i<=strlen(s_s); i++)
	{
		int ix;
		ix = num_from_char(s_s[strlen(s_s)-i]);

		carry = 0;

		for (j=1; j<=strlen(s_l); j++)
		{
			int iy = num_from_char(s_l[strlen(s_l)-j]);
			int iz = ix*iy + carry;

			if (iz > 9)
			{
				int tmp = iz % 10;
				carry = (iz - tmp) / 10;
				iz = tmp;
			} else carry = 0;		

			buf[j-1] = '0' + iz;
		}

		j = strlen(s_l);
		if (carry > 0) {
			buf[j] = '0' + carry;
			buf[j+1] = 0;
		} else buf[j] = 0;

		for (j=0; j<strlen(buf); j++)
			buf2[j] = buf[strlen(buf)-j-1];
		for (j=0; j<i-1; j++)
			buf2[strlen(buf)+j] = '0';
		buf2[strlen(buf)+i-1] = 0;
	
		ret2 = add_bignum(ret, buf2);
		free(ret);
		ret = ret2;

	}

	return ret;
}



/* Add a dword to a 64-bit value */
inline void _add_64bit(i64 *dest, const dword src)
{
	dword tmp = dest->lo + src;

	if (tmp < dest->lo) dest->hi++; /* "carry" */
	dest->lo = tmp;
}



/* note: output needs to be deallocated */
inline char *bignum_from_64bit(const i64 val)
{
	char *buf1, *buf2, *buf3;

	if (!val.hi) return bignum_from_int(val.lo);

	buf1 = bignum_from_int(val.hi);
        buf2 = mul_bignum(buf1, TWOPOW32);
        free(buf1);
        buf1 = bignum_from_int(val.lo);
	buf3 = add_bignum(buf1, buf2);
	free(buf1);
	free(buf2);
	
	return buf3;
}



inline double db_div_64_d(const i64 num, const dword denom)
{
	double a = (double)(num.lo) / (double)(denom);
	double b = (double)(num.hi) / (double)(denom);
	double c = (double)(4294967295.0) / (double)(denom);

	return a + b + c*(double)(num.hi);
}



inline int i_div_64_db(const i64 num, const double denom)
{
	double a = (double)(num.lo) / (denom);
	double b = (double)(num.hi) / (denom);
	double c = (double)(4294967295.0) / (denom);
	double d = a + b + c*(double)(num.hi);

	return (int)d;
}



inline int smaller_64(const i64 a, const i64 b)
{
	if (a.hi < b.hi)
		return 1;
	else if (a.hi > b.hi)
		return 0;
	else if (a.lo < b.lo)
		return 1;
	else
		return 0;
}



inline int bigger_64(const i64 a, const i64 b)
{
	if (a.hi > b.hi)
		return 1;
	else if (a.hi < b.hi)
		return 0;
	else if (a.lo > b.lo)
		return 1;
	else
		return 0;
}



inline i64 min_64(const i64 a, const i64 b)
{
	return smaller_64(a,b)?a:b;
}



inline i64 max_64(const i64 a, const i64 b)
{
	return bigger_64(a,b)?a:b;
}



/* note: free result */
inline char *add_commas(char *str, int want_free)
{
	char *str2, *p;
	int i,j,k;

	i = strlen(str);
	j = i + (i-1)/3;
	str2 = (char*)malloc(j + 1);
	p = str + i - 1;

	for (k=j; k>=0; k--)
		if ((j-k) % 4 == 0)
			str2[k] = ',';
		else {
			str2[k] = *p;
			p--;
		}

	str2[j] = 0;

	if (want_free) free(str);
	return str2;
}

#endif /* !HAVE_QUAD */

